%************************************************************************************************************************************************ 
%*************************************************** Model of human ventricular cardiomyocyte 2021 **********************************************
                                            %Electrophysiological activity at 3 Hz steady-state stimulation
                                                  %Published by Synkov at al. in Sci Rep 2021;11:3573   
%************************************************************************************************************************************************ 
                                                               %Compiled in MATLAB R2011b
%************************************************************************************************************************************************
                                                                  %ALL RIGHTS RESERVED
     %For research use only; commercial use prohibited. Distribution without permission of Michal Psek (mpasek@med.muni.cz) not permitted. 
%************************************************************************************************************************************************



%*****Global variables*****
global R T F Rot Rst Rel Vel 
global Smtot Sms Smt Cms Cmt
global Lt radiust ptcm To t0
global Nab Cab Kb ATPi Cap
global gNa gNas gNat gNaps gNapss gNapst gK1 gK1s gK1t gKp gKps gKpt gKto gKtos gKtot gKsNa gKsNas gKsNat gKatp gKatps gKatpt gNab gNabs gNabt gCab gCabs gCabt
global INaCamax INaCamaxs INaCamaxt INaKmax INaKmaxs INaKmaxt IpCamax IpCamaxs IpCamaxt 
global PCa PCas PCat PNal PNals PNalt PKl PKls PKlt PnsNa PnsNas PnsNat PnsK PnsKs PnsKt
global qKs qKss qKst qKr qKrs qKrt
global fNat fNapst fCat fKst fKrt fK1t fKpt fKsNat fnsNat fnsKt fKatpt fNabt fCabt fNaCat fNaKt fpCat fKtot fVs_t
global ICCC ICCT VCLAMP STIMULATION DIALYSIS



%Physical constants
R=8310; T=(273+37); F=96500;

%Resistence of extracellular solution [ohm*cm]
Rot=83.33;

%Total membrane area [cm^2]
Smtot=1.5e-4;

%Length of cell [cm]
%Lc=0.013;   

%Radius of cell [cm]
%rc=5.93e-4;

%T-tubule length [cm] 
Lt=5.015e-4;

%T-tubule radius [cm] 
radiust=210e-7;

%Density of T-tubules in surface membrane [n/cm^2]
ptcm=19000000;

%Bulk extracellular ion concentrations [mM]
Nab=140; Cab=2; Kb=5.4; 

%Concentration of Ca in electrode [mM]
Cap=0.0000005;

%Concentration of ATP in cytosole [mM]
ATPi=6.8;

%Conductivities, permeabilities or maximum current densities 
gNa=25;             %[mS/cm^2]             
gNaps=0.00102;      %[mS/cm^2]  
qKs=0.014;          %[mS/cm^2]     
qKr=0.2755;         %[mS/cm^2]     
gK1=0.6821;         %[mS/cm^2] 
gKp=0;              %[mS/cm^2]            
gKto=0.1323;        %[mS/cm^2]             
gKsNa=0.12848;      %[mS/cm^2]            
gNab=0.000564;      %[mS/cm^2]            
gCab=0.0011;        %[mS/cm^2]            
gKatp=0;            %[mS/cm^2]            
PCa=173e-5;         %[cm/s]                                                                                                                                          
PNal=0;             %[cm/s]                
PKl=32e-7;          %[cm/s]                 
PnsNa=1.75e-7;      %[cm/s]                
PnsK=1.75e-7;       %[cm/s]                
INaCamax=0.00015;   %[uA/cm^2/mM^4]       
INaKmax=1.17;       %[uA/cm^2]            
IpCamax=0.5;        %[uA/cm^2]               
 
%Fractions of ion transporters in T-tubular membrane [-]
fNat=0.557;   
fNapst=0.557;  
fCat=0.8;    
fKst=0.557;   
fKrt=0.557;   
fK1t=0.8;     
fKtot=0.557;  
fKpt=0.557;   
fKsNat=0.557; 
fnsNat=0.557; 
fnsKt=0.557;  
fNabt=0.557;  
fCabt=0.557;  
fNaCat=0.557; 
fNaKt=0.557;  
fpCat=0.2;    
fKatpt=0.557; 

%Fraction of Vs adjacent to T-tubular membrane [-]
fVs_t=0.51;  

%Serial resistance of electrode [Ohm]
Rel=2000000;

%Switching between current clamp and voltage clamp [0 and 1]
VCLAMP=0;

%Ion concentration changes in clefts [0-disabled, 1-allowed]
ICCC=1;

%Ion concentration changes in T-tubules [0 -disabled, 1 -allowed]
ICCT=1;

%Dialysis of cell by electrode [0 or 1]
DIALYSIS=0;

%Maximal integration step
Tinmax=0.1;

%Graphic output time
Graph_time=0;

%Assci file of final conditions [0 and 1]
Fincon=0;

%Assci file of results [0 and 1]
Results=0;



%Initial conditions for 3 Hz steady state (stabilised 1200 s)
ICon=[
 -8.3423693565616105e+001
  4.3397081401625745e-003
  7.7251745228563018e-001
  1.4468305641219249e-002
  3.2792361658791580e-008
  1.0273823097811511e-013
  3.1348225874236574e-006
  9.0136476069539972e-001
  2.0829890894796987e-002
  1.8051224719447959e-004
  6.9525984248348254e-007
  1.0042101705753758e-009
  7.5335406369528482e-011
  7.0892927093113026e-002
  6.5538473089889002e-003
  2.2720019823570107e-004
  3.5005025642435853e-006
  2.0224499186508658e-008
  1.4469021635947373e-011
  9.1890117749350342e-001
 -8.3423679263850858e+001
  4.3397164902727716e-003
  7.7251700558239000e-001
  1.4468688835984513e-002
  3.2792541954147361e-008
  9.9999686516847741e-001
  9.9878310000000003e-001
  5.1836701343054301e+000
  0.0000000000000000e+000
  8.9484604766009324e-001
  2.0679288755620254e-002
  1.7920755514857508e-004
  6.9023684514993649e-007
  9.9695941551603065e-010
  7.4790080795640127e-011
  7.6926357302568388e-002
  7.1116289785437409e-003
  2.4653692891074205e-004
  3.7984304075007436e-006
  2.1945831686969422e-008
  1.5700512859450430e-011
  9.1890117370871105e-001
  5.5341518102010143e+000
  1.3975362998762685e+002
  2.0284477836682728e+000
  1.3368132528108831e+002
  1.3338234874419523e+001
  2.4422279127464655e-004
  3.9421961079985694e-004
  9.9158156303100542e-001
  2.0707146438232987e-001
  1.3727036978598892e+000
  1.2547275687450692e+000
  1.4729675512148455e-001
  1.0492176992362744e-003
  1.3520172117064301e-007
  8.5165663864471786e-001
  4.9811691107872230e-002
  4.9812945752693460e-002
  4.5242494274409284e-001
  4.5242479534765778e-001
  9.6004062569961235e-001
  1.3180787387840251e-002
  6.7861694259314256e-005
  1.5530665746087875e-007
  1.5049848243193613e-010
  6.8723564798271838e-009
  1.4476636150288277e-009
  1.2351351239559749e-002
  9.5387734015564395e-003
  2.7625116683916975e-003
  3.5559469347623643e-004
  1.7180532742339092e-005
  4.8999509639784469e-004
 -3.4918411693969130e-019
 -4.7940900471267697e-021
 -2.4682500862929737e-023
 -5.6479340756549636e-026
 -4.8464455177510741e-029
  0.0000000000000000e+000
 -4.2426199261879593e-028
  9.6159884045294364e-001
  1.3202200427192659e-002
  6.7972041264286088e-005
  1.5555942716856727e-007
  1.5074387824131587e-010
  6.8836273023394919e-009
  1.4500273063116937e-009
  1.2371398416563871e-002
  9.5542697925911860e-003
  2.7670036822479084e-003
  3.5617344344278818e-004
  1.7208520853639405e-005
  4.9079935762561910e-004
  1.2769436627419322e-016
  1.7531702420026567e-018
  9.0262573752504944e-021
  2.0654240973884712e-023
  1.7738287414813731e-026
  1.4573207990162985e-025
  1.8010701883347645e-025
 -3.9773325976660995e-028
  2.0269326819311395e+000
  2.0254424095766717e+000
  2.0239771641083957e+000
  2.0225371623769397e+000
  2.0211226412081889e+000
  2.0197338581024420e+000
  2.0183710920136182e+000
  2.0170346442185969e+000
  5.5353092802921733e+000
  5.5364420408103943e+000
  5.5375507480540653e+000
  5.5386360650900670e+000
  5.5396986633336391e+000
  5.5407392243958506e+000
  5.5417584422571275e+000
  5.5427570258406389e+000
  5.5086862091089728e+000
  1.3979807863442770e+002
  2.0151704696194916e+000
  3.7037091230345638e-004
  1.2613936621503643e+000
  1.4867409691248437e-001
  1.0342939519943262e-003
  1.1054376602749442e-007
  8.5029150506438578e-001
  1.3368139205391157e+002
  1.3338262636173123e+001
  2.0266683798231814e-004
  1.3368137088204918e+002
  1.3338290286688883e+001
  2.2499476680845401e-004
 ];


Vms0=ICon(1);
ms0=ICon(2);
hs0=ICon(3);
xss0=ICon(4);
r2s0=ICon(5);
r3s0=ICon(6);
r4s0=ICon(7);
Cst0=ICon(8);
C10=ICon(9);
C20=ICon(10);
C30=ICon(11);
C40=ICon(12);
Co0=ICon(13);
Ccast0=ICon(14);
Cca10=ICon(15);
Cca20=ICon(16);
Cca30=ICon(17);
Cca40=ICon(18);
Ccao0=ICon(19);
ys0=ICon(20);
Vmt0=ICon(21);
mt0=ICon(22);
ht0=ICon(23);
xst0=ICon(24);
r2t0=ICon(25);
r3t0=ICon(26);
r4t0=ICon(27);
dt0=ICon(28);
ft0=ICon(29);
TCst0=ICon(30);
TC10=ICon(31);
TC20=ICon(32);
TC30=ICon(33);
TC40=ICon(34);
TCo0=ICon(35);
TCcast0=ICon(36);
TCca10=ICon(37);
TCca20=ICon(38);
TCca30=ICon(39);
TCca40=ICon(40);
TCcao0=ICon(41);
yt0=ICon(42);
Kt_10=ICon(43);
Nat0=ICon(44);
Cat_10=ICon(45);
Ki0=ICon(46);
Nai0=ICon(47);
Cai0=ICon(48);
Cadt0=ICon(49);
BTRH0=ICon(50);
BTRL0=ICon(51);
CaSRup0=ICon(52);
CaSRrel_t0=ICon(53);
F1t0=ICon(54);
F2t0=ICon(55);
F3t0=ICon(56);
F4t0=ICon(57);
xrs0=ICon(58);
xrt0=ICon(59);
xris0=ICon(60);
xrit0=ICon(61);
C0_Na0=ICon(62);
C1_Na0=ICon(63);
C2_Na0=ICon(64);
C3_Na0=ICon(65);
C4_Na0=ICon(66);
O1_Na0=ICon(67);
O2_Na0=ICon(68);
IC0_Na0=ICon(69);
IC1_Na0=ICon(70);
IC2_Na0=ICon(71);
IC3_Na0=ICon(72);
IC4_Na0=ICon(73);
IO1_Na0=ICon(74);
MC0_Na0=ICon(75);
MC1_Na0=ICon(76);
MC2_Na0=ICon(77);
MC3_Na0=ICon(78);
MC4_Na0=ICon(79);
MO1_Na0=ICon(80);
MO2_Na0=ICon(81);
TC0_Na0=ICon(82);
TC1_Na0=ICon(83);
TC2_Na0=ICon(84);
TC3_Na0=ICon(85);
TC4_Na0=ICon(86);
TO1_Na0=ICon(87);
TO2_Na0=ICon(88);
TIC0_Na0=ICon(89);
TIC1_Na0=ICon(90);
TIC2_Na0=ICon(91);
TIC3_Na0=ICon(92);
TIC4_Na0=ICon(93);
TIO1_Na0=ICon(94);
TMC0_Na0=ICon(95);
TMC1_Na0=ICon(96);
TMC2_Na0=ICon(97);
TMC3_Na0=ICon(98);
TMC4_Na0=ICon(99);
TMO1_Na0=ICon(100);
TMO2_Na0=ICon(101);
MO1_Na02=ICon(102);
Cat_20=ICon(103);
Cat_30=ICon(104);
Cat_40=ICon(105);
Cat_50=ICon(106);
Cat_60=ICon(107);
Cat_70=ICon(108);
Cat_80=ICon(109);
Cat_90=ICon(110);
Kt_20=ICon(111);
Kt_30=ICon(112);
Kt_40=ICon(113);
Kt_50=ICon(114);
Kt_60=ICon(115);
Kt_70=ICon(116);
Kt_80=ICon(117);
Kt_90=ICon(118);
Ke0=ICon(119);
Nae0=ICon(120);
Cae0=ICon(121);
Cads0=ICon(122);
CaSRrel_s0=ICon(123);
F1s0=ICon(124);
F2s0=ICon(125);
F3s0=ICon(126);
F4s0=ICon(127);
Kss0=ICon(128); 
Nass0=ICon(129); 
Cass0=ICon(130);
Kst0=ICon(131); 
Nast0=ICon(132); 
Cast0=ICon(133);

RMT=[];



%***********************  Solution 1  ************************
%Command voltage (only for VCLAMP=1)
Vel=-80;
%Change of external/bulk ion concentrations
Kb=5.4; Cab=2; Nab=140;
%Stimulation option (only for VCLAMP=0)
STIMULATION=1; 
%First interval of simulation
Tstart=0; Tend=0.3333; To=Tend-Tstart;
%Parameters of integration 
InitialStep=0.0001; MaxStep=Tinmax; TOL=1e-6; trace=0;
%Simulation
t0=Tstart; tend=Tend; graph


%***********************  Solution 2  ************************
%Command voltage (only for VCLAMP=1)
Vel=0;
%Change of external/bulk ion concentrations
Kb=5.4; Cab=2;
%Stimulation option (only for VCLAMP=0)
STIMULATION=1;
%Interval of simulation
Tstart=0.3333; Tend=0.6666;
%Stimulation period 
To=0.3333;
for I=Tstart:To:Tend;
%Parameters of integration 
InitialStep=0.0001; MaxStep=Tinmax; TOL=1e-6; trace=0;
%Simulation
I
t0=I; tend=I+To; graph
end



RC=[
Vms(length(Vms));
ms(length(ms));
hs(length(hs));
xss(length(xss));
r2s(length(r2s));
r3s(length(r3s));
r4s(length(r4s));
Cst(length(Cst));
C1(length(C1));
C2(length(C2));
C3(length(C3));
C4(length(C4));
Co(length(Co));
Ccast(length(Ccast));
Cca1(length(Cca1));
Cca2(length(Cca2));
Cca3(length(Cca3));
Cca4(length(Cca4));
Ccao(length(Ccao));
ys(length(ys));
Vmt(length(Vmt));
mt(length(mt));
ht(length(ht));
xst(length(xst));
r2t(length(r2t));
r3t(length(r3t));
r4t(length(r4t));
dt(length(dt));
ft(length(ft));
TCst(length(TCst));
TC1(length(TC1));
TC2(length(TC2));
TC3(length(TC3));
TC4(length(TC4));
TCo(length(TCo));
TCcast(length(TCcast));
TCca1(length(TCca1));
TCca2(length(TCca2));
TCca3(length(TCca3));
TCca4(length(TCca4));
TCcao(length(TCcao));
yt(length(yt));
Kt_1(length(Kt_1));
Nat(length(Nat));
Cat_1(length(Cat_1));
Ki(length(Ki));
Nai(length(Nai));
Cai(length(Cai));
Cadt(length(Cadt));
BTRH(length(BTRH));
BTRL(length(BTRL));
CaSRup(length(CaSRup));
CaSRrel_t(length(CaSRrel_t));
F1t(length(F1t));
F2t(length(F2t));
F3t(length(F3t));
F4t(length(F4t));
xrs(length(xrs));
xrt(length(xrt));
xris(length(xris));
xrit(length(xrit));
C0_Na(length(C0_Na));
C1_Na(length(C1_Na));
C2_Na(length(C2_Na));
C3_Na(length(C3_Na));
C4_Na(length(C4_Na));
O1_Na(length(O1_Na));
O2_Na(length(O2_Na));
IC0_Na(length(IC0_Na));
IC1_Na(length(IC1_Na));
IC2_Na(length(IC2_Na));
IC3_Na(length(IC3_Na));
IC4_Na(length(IC4_Na));
IO1_Na(length(IO1_Na));
MC0_Na(length(MC0_Na));
MC1_Na(length(MC1_Na));
MC2_Na(length(MC2_Na));
MC3_Na(length(MC3_Na));
MC4_Na(length(MC4_Na));
MO1_Na(length(MO1_Na));
MO2_Na(length(MO2_Na));
TC0_Na(length(TC0_Na));
TC1_Na(length(TC1_Na));
TC2_Na(length(TC2_Na));
TC3_Na(length(TC3_Na));
TC4_Na(length(TC4_Na));
TO1_Na(length(TO1_Na));
TO2_Na(length(TO2_Na));
TIC0_Na(length(TIC0_Na));
TIC1_Na(length(TIC1_Na));
TIC2_Na(length(TIC2_Na));
TIC3_Na(length(TIC3_Na));
TIC4_Na(length(TIC4_Na));
TIO1_Na(length(TIO1_Na));
TMC0_Na(length(TMC0_Na));
TMC1_Na(length(TMC1_Na));
TMC2_Na(length(TMC2_Na));
TMC3_Na(length(TMC3_Na));
TMC4_Na(length(TMC4_Na));
TMO1_Na(length(TMO1_Na));
TMO2_Na(length(TMO2_Na));
MO1_Na2(length(MO1_Na2));
Cat_2(length(Cat_2));
Cat_3(length(Cat_3));
Cat_4(length(Cat_4));
Cat_5(length(Cat_5));
Cat_6(length(Cat_6));
Cat_7(length(Cat_7));
Cat_8(length(Cat_8));
Cat_9(length(Cat_9));
Kt_2(length(Kt_2));
Kt_3(length(Kt_3));
Kt_4(length(Kt_4));
Kt_5(length(Kt_5));
Kt_6(length(Kt_6));
Kt_7(length(Kt_7));
Kt_8(length(Kt_8));
Kt_9(length(Kt_9));
Ke(length(Ke));
Nae(length(Nae));
Cae(length(Cae));
Cads(length(Cads));
CaSRrel_s(length(CaSRrel_s));
F1s(length(F1s));
F2s(length(F2s));
F3s(length(F3s));
F4s(length(F4s));
Kss(length(Kss));
Nass(length(Nass));
Cass(length(Cass));
Kst(length(Kst));
Nast(length(Nast));
Cast(length(Cast));
];

if Fincon==1
save fincon3Hz.asc RC -ascii -double;
end;

if Results==1
save results3Hz.asc RMT -ascii;
end;